package com.nisum.example.mongodb.service;

import com.nisum.example.mongodb.model.Employee;
import com.nisum.example.mongodb.repository.EmployeeRepository;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;
import reactor.test.StepVerifier;

import static org.junit.jupiter.api.Assertions.assertEquals;

@SpringBootTest
public class EmployeeServiceTest {

    @Autowired
    private IEmployeeService employeeService;

    @MockBean
    private EmployeeRepository employeeRepository;

    @Test
    void testInsertEmployee(){
        Employee employee=new Employee(101,"SYED",1500L);
        Mono<Employee> employeeMono=Mono.just(employee);
        Mockito.when(employeeRepository.save(employee)).thenReturn(employeeMono);
        assertEquals(employeeMono,employeeService.insert(employee));
        StepVerifier.create(employeeService.insert(employee).log())
                .expectNext(employee)
                .expectComplete()
                .verify();

    }

    @Test
    void testEmployeeFetchById(){
        Employee employee=new Employee(101,"SYED",1500L);
        Mono<Employee> employeeMono=Mono.just(employee);
        Mockito.when(employeeRepository.findById(101)).thenReturn(employeeMono);
        assertEquals(employeeMono,employeeService.fetchById(101));
        StepVerifier.create(employeeService.fetchById(101).log())
                .expectNext(employee)
                .expectComplete()
                .verify();

    }

    @Test
    void testEmployeeFetchByName(){
        Employee employee=new Employee(101,"SYED",1500L);

        Flux<Employee> employeeFlux=Flux.just(employee);
        Mockito.when(employeeRepository.findByName("SYED")).thenReturn(employeeFlux);
        assertEquals(employeeFlux,employeeService.fetchByName("SYED"));
        StepVerifier.create(employeeService.fetchByName("SYED").log())
                .expectNext(employee)
                .expectComplete()
                .verify();

    }

    @Test
    void testFetchAllEmployees(){
        Employee employee=new Employee(101,"SYED",1500L);
        Flux<Employee> employeeFlux=Flux.just(employee);
        Mockito.when(employeeRepository.findAll()).thenReturn(employeeFlux);
        assertEquals(employeeFlux,employeeService.fetchAll());
        StepVerifier.create(employeeService.fetchAll())
                .expectNext(employee)
                .expectComplete()
                .verify();
    }

    @Test
    void testModifyEmployee(){
        Employee employee=new Employee(101,"SYED",1500L);
        Mono<Employee> employeeMono=Mono.just(employee);
        Mockito.when(employeeRepository.save(employee)).thenReturn(employeeMono);
        assertEquals(employeeMono,employeeService.modify(employee));
        StepVerifier.create(employeeService.modify(employee))
                .expectNext(employee)
                .expectComplete()
                .verify();
    }

    @Test
    void testRemoveByIdEmployee(){
        Mono<Void> voidMono=Mono.empty();
        Mockito.when(employeeRepository.deleteById(1)).thenReturn(voidMono);
        assertEquals(voidMono,employeeService.removeById(1));
        StepVerifier.create(employeeService.removeById(1))
                .expectComplete()
                .verify();

    }


}
